@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugin.h,v 1.11 2005/12/12 17:27:55 pborelli Exp $
+ * $Id: gedit-plugin.h,v 1.12 2006/01/06 16:45:20 pborelli Exp $
  */
 
 #ifndef __GEDIT_PLUGIN_H__
@@ -80,14 +80,19 @@ struct _GeditPluginClass
 	void 		(*update_ui)		(GeditPlugin *plugin,
 						 GeditWindow *window);
 
-	GtkWidget      *(*create_configure_dialog)
+	GtkWidget 	*(*create_configure_dialog)
+						(GeditPlugin *plugin);
+
+	/* Plugins should not override this, it's handled automatically by
+	   the GeditPluginClass */
+	gboolean 	(*is_configurable)
 						(GeditPlugin *plugin);
 
 	/* Padding for future expansion */
 	void		(*_gedit_reserved1)	(void);
 	void		(*_gedit_reserved2)	(void);
 	void		(*_gedit_reserved3)	(void);
-	void		(*_gedit_reserved4)	(void);						
+	void		(*_gedit_reserved4)	(void);
 };
 
 /*
