@@ -1236,6 +1236,11 @@ DEFINE_BOOL_PREF (enable_syntax_highlighting,
 		  GPM_SYNTAX_HL_ENABLE,
 		  GPM_DEFAULT_SYNTAX_HL_ENABLE)
 
+/* Enable search highlighting */
+DEFINE_BOOL_PREF (enable_search_highlighting,
+		  GPM_SEARCH_HIGHLIGHTING_ENABLE,
+		  GPM_DEFAULT_SEARCH_HIGHLIGHTING_ENABLE)
+
 
 GSList *
 gedit_prefs_manager_get_writable_vfs_schemes (void)
@@ -1275,6 +1280,15 @@ gedit_prefs_manager_get_writable_vfs_schemes (void)
 	return strings;
 }
 
+gboolean
+gedit_prefs_manager_get_restore_cursor_position (void)
+{
+	gedit_debug (DEBUG_PREFS);
+
+	return gedit_prefs_manager_get_bool (GPM_RESTORE_CURSOR_POSITION,
+					     GPM_DEFAULT_RESTORE_CURSOR_POSITION);
+}
+
 /* The following functions are taken from gconf-client.c 
  * and partially modified. 
  * The licensing terms on these is: 
