@@ -19,6 +19,7 @@
 import ElementTree as et
 import os
 import gtk
+from gtk import gdk
 import gnomevfs
 import gedit
 from gettext import gettext as _
@@ -35,39 +36,34 @@ class ToolsTree:
 			os.mkdir(self.xml_location)
 
 		self.xml_location = os.path.join(self.xml_location, 'gedit-tools.xml')
-
-		if not os.path.isfile(self.xml_location):
-			self.tree = et.parse(self.get_stock_file())
-			self.root = self.tree.getroot()
-			self.save()
-		else:
-			try:
+		
+		try:
+			if not os.path.isfile(self.xml_location):
+				self.tree = et.parse(self.get_stock_file())
+			else:
 				self.tree = et.parse(self.xml_location)
-				self.root = self.tree.getroot()
-			except:
-				self.tree = None
-				self.root = None
-
+			self.root = self.tree.getroot()
+		except:
+			self.root = et.Element('tools')
+			self.tree = et.ElementTree(self.root)
+	
 	def __iter__(self):
 		return iter(self.root)
-
+	
 	def get_stock_file(self):
-		if 'XDG_DATA_DIRS' in os.environ:
-			dirs = os.environ['XDG_DATA_DIRS']
-
-		if not dirs:
-			dirs = '/usr/local/share:/usr/share'
-
-		dirs = dirs.split(':')
-
+		dirs = os.getenv('XDG_DATA_DIR')
+		if dirs:
+			dirs = dirs.split(os.pathsep)
+		else:
+			dirs = ('/usr/local/share', '/usr/share')
+		
 		for d in dirs:
-			f = d + '/gedit-2/plugins/externaltools/stock-tools.xml'
+			f = os.path.join(d, 'gedit-2/plugins/externaltools/stock-tools.xml')
 			if os.path.isfile(f):
 				return f
 
-		# print 'not found'
 		return None
-
+	
 	def save(self):
 		self.tree.write(self.xml_location)
 		
@@ -77,7 +73,7 @@ class ToolsTree:
 		
 		for tool in self.root:
 			if tool != ignore:
-				skey, smod = gtk.accelerator_parse(tool.get('accelerator'))
+				skey, smod = gtk.accelerator_parse(default(tool.get('accelerator'), ''))
 				if skey == keyval and smod & mod == smod:
 					return tool
 		return None
@@ -90,8 +86,10 @@ def default(val, d):
 		return d
 
 # ==== UI related functions ====
+APPLICABILITIES = ('all', 'titled', 'local', 'remote', 'untitled')
+
 def insert_tools_menu(window, tools = None):
-	window_data = dict()	
+	window_data = dict()
 	window.set_data("ToolsPluginCommandsData", window_data)
 
 	if tools is None:
@@ -99,20 +97,24 @@ def insert_tools_menu(window, tools = None):
 
 	manager = window.get_ui_manager()
 
-	window_data['action_group'] = gtk.ActionGroup("GeditToolsPluginCommandsActions")
-	window_data['action_group'].set_translation_domain('gedit')
+	window_data['action_groups'] = dict()
 	window_data['ui_id'] = manager.new_merge_id()
 
 	i = 0;
 	for tool in tools:
 		menu_id = "ToolCommand%06d" % i
-		window_data['action_group'].add_actions([(menu_id,
-		                                          None,
-		                                          tool.get('label'),
-		                                          tool.get('accelerator'),
-		                                          tool.get('description'),
-		                                          capture_menu_action)],
-		                                        (window, tool))
+		
+		ap = tool.get('applicability')
+		if ap not in window_data['action_groups']:
+			window_data['action_groups'][ap] = \
+			    gtk.ActionGroup("GeditToolsPluginCommandsActions%s" % ap.capitalize())
+			window_data['action_groups'][ap].set_translation_domain('gedit')
+		
+		window_data['action_groups'][ap].add_actions(
+			[(menu_id, None, tool.get('label'),
+			  tool.get('accelerator'), tool.get('description'),
+			  capture_menu_action)],
+			(window, tool))
 		manager.add_ui(window_data['ui_id'],
 		               '/MenuBar/ToolsMenu/ToolsOps_4',
 		               menu_id, 
@@ -121,63 +123,101 @@ def insert_tools_menu(window, tools = None):
 		               False)
 		i = i + 1
 
-	manager.insert_action_group(window_data['action_group'], -1)
+	for applic in APPLICABILITIES:
+		if applic in window_data['action_groups']:
+			manager.insert_action_group(window_data['action_groups'][applic], -1)
 
 def remove_tools_menu(window):
 	window_data = window.get_data("ToolsPluginCommandsData")
 
 	manager = window.get_ui_manager()
 	manager.remove_ui(window_data['ui_id'])
-	manager.remove_action_group(window_data['action_group'])
+	for action_group in window_data['action_groups'].itervalues():
+		manager.remove_action_group(action_group)
 	window.set_data("ToolsPluginCommandsData", None)
 
 def update_tools_menu(tools = None):
 	for window in gedit.gedit_app_get_default().get_windows():
 		remove_tools_menu(window)
 		insert_tools_menu(window, tools)
+		filter_tools_menu(window)
 		window.get_ui_manager().ensure_update()
 
+def filter_tools_menu(window):
+	action_groups = window.get_data("ToolsPluginCommandsData")['action_groups']
+
+	document = window.get_active_document()
+	if document is not None:
+		active_groups = ['all']
+		uri = document.get_uri()
+		if uri is not None:
+			active_groups.append('titled')
+			if gnomevfs.get_uri_scheme(uri) == 'file':
+				active_groups.append('local')
+			else:
+				active_groups.append('remote')
+		else:
+			active_groups.append('untitled')
+	else:
+		active_groups = []
+
+	for name, group in action_groups.iteritems():
+		group.set_sensitive(name in active_groups)
+
 # ==== Capture related functions ====
 def capture_menu_action(action, window, node):
+
 	# Get the panel
 	panel = window.get_data("ToolsPluginWindowData")["output_buffer"]
 	panel.show()
 	panel.clear()
 
-	document = window.get_active_document()
-	if document is None:
-		# :TODO: Allow command on no document
-		panel.write("No document\n", panel.command)
-		return
+	# Configure capture environment
+ 	capture = Capture(node.text, os.getcwd())
+ 	capture.env = os.environ.copy()
+	capture.set_env(GEDIT_CWD = os.getcwd())
 
-	uri = document.get_uri()
-	if uri is None:
-		# :TODO: Allow command on unnamed documents
-		panel.write("Current document has no file name\n", panel.command)
-		return
+	view = window.get_active_view()
+	if view is not None:
+		# Environment vars relative to current document
+		document = view.get_buffer()
+		uri = document.get_uri()
+		if uri is not None:
+			scheme = gnomevfs.get_uri_scheme(uri)
+			name = os.path.basename(uri)
+			capture.set_env(GEDIT_CURRENT_DOCUMENT_URI    = uri,
+			                GEDIT_CURRENT_DOCUMENT_NAME   = name,
+			                GEDIT_CURRENT_DOCUMENT_SCHEME = scheme)
+			if scheme == 'file':
+		 		path = gnomevfs.get_local_path_from_uri(uri)
+				cwd = os.path.dirname(path)
+				capture.set_cwd(cwd)
+				capture.set_env(GEDIT_CURRENT_DOCUMENT_PATH = path,
+				                GEDIT_CURRENT_DOCUMENT_DIR  = cwd)
+		
+		documents_uri = [document.get_uri()
+		                         for document in window.get_documents()
+		                         if document.get_uri() is not None]
+		documents_path = [gnomevfs.get_local_path_from_uri(uri)
+		                         for uri in documents_uri
+		                         if gnomevfs.get_uri_scheme(uri) == 'file']
+		capture.set_env(GEDIT_DOCUMENTS_URI  = ' '.join(documents_uri),
+		                GEDIT_DOCUMENTS_PATH = ' '.join(documents_path))
 
-	# Configure capture environment	
-	path = gnomevfs.get_local_path_from_uri(uri)
- 	cwd = os.path.dirname(path)
-
- 	capture = Capture(node.text, cwd)
  	capture.set_flags(capture.CAPTURE_BOTH)
- 	capture.env = {
- 		'GEDIT_CURRENT_DOCUMENT_URI' : uri,
- 		'GEDIT_CURRENT_DOCUMENT_PATH': path,
- 		'GEDIT_CURRENT_DOCUMENT_DIR' : cwd
- 	}
 
 	# Assign the error output to the output panel
 	panel.process = capture
 	
 	capture.connect('stderr-line',   capture_stderr_line_panel,   panel)
 	capture.connect('begin-execute', capture_begin_execute_panel, panel, node.get('label'))
-	capture.connect('end-execute',   capture_end_execute_panel,   panel)
+	capture.connect('end-execute',   capture_end_execute_panel,   panel, view)
 
 	# Get input text
-	input_type = node.get('input')
-	if input_type != 'nothing':
+	input_type = default(node.get('input'), 'nothing')
+	output_type = default(node.get('output'), 'output-panel')
+
+	if input_type != 'nothing' and view is not None:
 		if input_type == 'document':
 			start, end = document.get_bounds()
 		elif input_type == 'selection':
@@ -185,6 +225,8 @@ def capture_menu_action(action, window, node):
 				start, end = document.get_selection_bounds()
 			except ValueError:
 				start, end = document.get_bounds()
+				if output_type == 'replace-selection':
+					document.select_range(start, end)
 		elif input_type == 'line':
 			start = document.get_insert()
 			end = insert.copy()
@@ -208,27 +250,38 @@ def capture_menu_action(action, window, node):
 		capture.set_input(input_text)
 	
 	# Assign the standard output to the chosen "file"
-	output_type = node.get('output')
-	if output_type == 'output-panel':
-		capture.connect('stdout-line', capture_stdout_line_panel, panel)
-	else:
+	if output_type == 'new-document':
+		tab = window.create_tab(True)
+		view = tab.get_view()
+		document = tab.get_document()
+		pos = document.get_start_iter()
+		capture.connect('stdout-line', capture_stdout_line_document, document, pos)
+		document.begin_user_action()
+		view.set_editable(False)
+		view.set_cursor_visible(False)
+	elif output_type != 'output-panel' and view is not None:
+		document.begin_user_action()
+		view.set_editable(False)
+		view.set_cursor_visible(False)
+		
 		if output_type == 'insert':
 			pos = document.get_iter_at_mark(document.get_mark('insert'))
 		elif output_type == 'replace-selection':
-			document.delete_selection()
+			document.delete_selection(False, False)
 			pos = document.get_iter_at_mark(document.get_mark('insert'))
-		elif output_type == 'new-document':
-			document = window.create_tab(True).get_document()
-			pos = document.get_end_iter()
 		elif output_type == 'replace-document':
 			document.set_text('')
 			pos = document.get_end_iter()
 		else:
 			pos = document.get_end_iter()
 		capture.connect('stdout-line', capture_stdout_line_document, document, pos)
+	else:
+		capture.connect('stdout-line', capture_stdout_line_panel, panel)
+		document.begin_user_action()
+
 	
 	# Run the command
-	document.begin_user_action()
+	view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gdk.Cursor(gdk.WATCH))
 	capture.execute()
 	document.end_user_action()
 
@@ -240,9 +293,12 @@ def capture_begin_execute_panel(capture, panel, label):
 	panel.clear()
 	panel.write("Running %s...\n" % label, panel.command_tag)
 
-def capture_end_execute_panel(capture, exit_code, panel):
+def capture_end_execute_panel(capture, exit_code, panel, view):
 	panel.write("Exit code: %s\n" % exit_code, panel.command_tag)
 	panel['stop'].set_sensitive(False)
+	view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gdk.Cursor(gdk.XTERM))
+	view.set_cursor_visible(True)
+	view.set_editable(True)
 
 def capture_stdout_line_panel(capture, line, panel):
 	panel.write(line)
