@@ -33,7 +33,7 @@ class UniqueById:
 	__shared_state = WeakKeyDictionary()
 	
 	def __init__(self, i):
-		if self.__class__.__shared_state.has_key(i):
+		if i in self.__class__.__shared_state:
 			self.__dict__ = self.__class__.__shared_state[i]
 			return True
 		else:
@@ -94,5 +94,6 @@ class OutputPanel(UniqueById):
 		gobject.idle_add(self.scroll_to_end)
 
 	def show(self):
-		# :TODO: Show the good panel
-		pass
+		panel = self.window.get_bottom_panel()
+		panel.show()
+		panel.activate_item(self.panel)
